# Adding and Updating Custom Databases

You can add or update custom reference databases using Toolchest – the interface is the same as running any tool, 
except `inputs` is replaced by `database_path`.

Adding and updating databases function as [Async Runs](async-runs.md). The `add_database` and 
`update_database` functions return after transferring the data with:

- a unique ID that you can use with `.get_status()` to track status
- the new `database_name` and `database_version`

##  Adding a Custom Database

You can add a custom database for any tool that exists in Toolchest. 

The custom database must already be generated for the specific tool. For example, the custom database for Kraken 2 must 
be generated by `kraken2-build` rather than a collection of FASTQ files.

Arguments:

- `database_path`:  Path or list of paths (local or S3) to file(s) containing the custom database.
- `tool`: Toolchest tool class with which you use the database (e.g. `toolchest.tools.DiamondBlastp`, 
`toolchest.tools.Kraken2`).
- `database_name`: name of the new custom database.
- `database_primary_name`: If you are uploading multiple database files and your tool takes in a certain file or prefix 
**instead of the whole directory** as its command-line argument, use this to specify the name of that file or prefix.

  (For example, if you are uploading files `database.file.1` and `database.file.2`for a tool that takes in 
`/path/to/database/database.file` as its command-line database, use `database_primary_name=database.file`.)

The return is a `Toolchest.api.Output` object, containing:

- `database_name`
- `database_version`
- `run_id`

Here's an example of adding a custom database for Kraken 2 using an S3 URI:

```python
import toolchest_client as tc

tc.set_key("YOUR_KEY")

tc.update_database(
  database_path=[
    "s3://toolchest-integration-tests/arbitrary_directory/db.files.1",
    "s3://toolchest-integration-tests/arbitrary_directory/db.files.2",
  ]
  tool=tc.tools.Kraken2,
  database_name="standard",
  database_primary_name="db.files",
)
```



Note that it may take up to 1 hour for the custom database to be ready to use.

##  Updating a Custom Database

You can create a new custom version for any tool and database in Toolchest. This is very similar to adding a custom 
database, except the `database_name` for the database must already exist.

Arguments:

- `database_path`:  Path or list of paths (local or S3) to file(s) containing the custom database.
- `tool`: Toolchest tool class with which you use the database (e.g. `toolchest.tools.DiamondBlastp`, 
`toolchest.tools.Kraken2`).
- `database_name`: name of the _existing_ database
- `database_primary_name`: If you are uploading multiple database files and your tool takes in a certain file or prefix 
**instead of the whole directory** as its command-line argument, use this to specify the name of that file or prefix.

  (For example, if you are uploading files `database.file.1` and `database.file.2`for a tool that takes in 
`/path/to/database/database.file` as its command-line database, use `database_primary_name=database.file`.)

Returns a Toolchest.api.Output object, containing:

- `database_name`
- `database_version` (auto-incremented from the latest version)
- `run_id`

Here's an example update of the standard Kraken 2 database:

```python
import toolchest_client as tc

tc.set_key("YOUR_KEY")

tc.update_database(
  database_path=[
    "s3://toolchest-integration-tests/arbitrary_directory/db.files.1",
    "s3://toolchest-integration-tests/arbitrary_directory/db.files.2",
  ],
  tool=tc.tools.Kraken2,
  database_name="standard",
  database_primary_name="db.files",
)
```



Note that it may take up to 1 hour after the transfer is complete for the custom database to be ready to use.

## Security

By default, the privacy setting of all custom databases is the equivalent of "unlisted". This means that if anybody 
else knows the name and version of your database, they can access it.

We support fully private custom databases as a part of the managed-hosted and on-prem versions of Toolchest.